# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from designate.objects.adapters.yaml import base
from designate import objects


class PoolNameserverYAMLAdapter(base.YAMLAdapter):

    ADAPTER_OBJECT = objects.PoolNameserver

    MODIFICATIONS = {
        'fields': {
            'host': {
                'read_only': False
            },
            'port': {
                'read_only': False
            }
        }
    }


class PoolNameserverListYAMLAdapter(base.YAMLAdapter):

    ADAPTER_OBJECT = objects.PoolNameserverList

    MODIFICATIONS = {}
